﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.Runtime;

namespace WorkflowActivity
{
    [ActivityValidator(typeof(CreateUserActivityValidator)), DefaultEvent("MethodInvoking")]
    public partial class CreateUserActivity : System.Workflow.ComponentModel.Activity
    {
        public CreateUserActivity()
        {
            InitializeComponent();
        }

        public static DependencyProperty MethodInvokingEvent =
            DependencyProperty.Register("MethodInvoking", typeof(EventHandler), typeof(CreateUserActivity));

        public event EventHandler MethodInvoking
        {
            add { base.AddHandler(MethodInvokingEvent, value); }
            remove { base.RemoveHandler(MethodInvokingEvent, value); }
        }

        public static DependencyProperty MethodInvokedEvent =
            DependencyProperty.Register("MethodInvoked", typeof(EventHandler), typeof(CreateUserActivity));

        public event EventHandler MethodInvoked
        {
            add { base.AddHandler(MethodInvokedEvent, value); }
            remove { base.RemoveHandler(MethodInvokedEvent, value); }
        }

        public static DependencyProperty UserNameProperty =
            DependencyProperty.Register("UserName", typeof(string), typeof(CreateUserActivity));

        public string UserName
        {
            get { return ((string)base.GetValue(UserNameProperty)); }
            set { base.SetValue(UserNameProperty, value); }
        }

        public static DependencyProperty ServerProperty =
            DependencyProperty.Register("Server", typeof(string), typeof(CreateUserActivity));

        public string Server
        {
            get { return ((string)base.GetValue(ServerProperty)); }
            set { base.SetValue(ServerProperty, value); }
        }

        public static DependencyProperty OUProperty =
            DependencyProperty.Register("OU", typeof(string), typeof(CreateUserActivity));

        public string OU
        {
            get { return ((string)base.GetValue(OUProperty)); }
            set { base.SetValue(OUProperty, value); }
        }

        public static DependencyProperty PasswordProperty =
            DependencyProperty.Register("Password", typeof(string), typeof(CreateUserActivity));
        public string Password
        {
            get { return ((string)base.GetValue(PasswordProperty)); }
            set { base.SetValue(PasswordProperty, value); }
        }

        public static DependencyProperty EmailProperty =
            DependencyProperty.Register("Email", typeof(string), typeof(CreateUserActivity));
        public string Email
        {
            get { return ((string)base.GetValue(EmailProperty)); }
            set { base.SetValue(EmailProperty, value); }

        }

        public static DependencyProperty GroupsProperty =
            DependencyProperty.Register("Groups", typeof(string), typeof(CreateUserActivity));

        public string Groups
        {
            get { return ((string)base.GetValue(GroupsProperty)); }
            set { base.SetValue(GroupsProperty, value); }
        }

        public static DependencyProperty AttributesProperty =
            DependencyProperty.Register("Attributes", typeof(string[][]), typeof(CreateUserActivity));

        public string[][] Attributes
        {
            get { return ((string[][])base.GetValue(AttributesProperty)); }
            set { base.SetValue(AttributesProperty, value); }
        }

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            try
            {
                base.RaiseEvent(MethodInvokingEvent, this, EventArgs.Empty);
                if (!string.IsNullOrEmpty(Server) && !string.IsNullOrEmpty(OU) && !string.IsNullOrEmpty(UserName))
                {
                    // We have enough data to proceed
                    using (LDAP ldap = new LDAP(Server, true))
                    {
                        if (string.IsNullOrEmpty(Password)) Password = "P@ssw0rd";
                        if (Groups == null)
                        {
                            Groups = "Domain Users";
                        }

                        Dictionary<string, string> attrs = null;
                        if (Attributes != null)
                        {
                            attrs = new Dictionary<string, string>();
                            foreach (string[] attr in Attributes)
                            {
                                attrs.Add(attr[0], attr[1]);
                            }
                        }

                        ldap.CreateUser(OU, OU, UserName, Password, Email, AdsUserFlags.NormalAccount, 
                                        Groups.Split(new char[] {';'}), attrs);
                    }
                }
                base.RaiseEvent(MethodInvokedEvent, this, EventArgs.Empty);
            }
            catch (Exception excpt)
            {
                System.Diagnostics.Trace.WriteLine(excpt.ToString());
                throw;
            }

            return (ActivityExecutionStatus.Closed);
        }

    }
}
